@@ -19,18 +19,24 @@
  */
 
 #include "BSEQuencer.hpp"
+#include <stdexcept>
+#include <ctime>
+#include "BUtilities/stof.hpp"
 
 BSEQuencer::BSEQuencer (double samplerate, const LV2_Feature* const* features) :
 	map (NULL), unmap (NULL), inputPort (NULL), outputPort (NULL),
 	output_forge (), output_frame (),
 	new_controllers {nullptr}, controllers {0},
-	rate (samplerate), bpm (120.0f), beatsPerBar (4.0f),
+	rate (samplerate), bpm (120.0f), beatsPerBar (4.0f), speed (0.0f),
 	outCapacity (0), position (0.0),
 	ui_on (false), scheduleNotifyPadsToGui (false), scheduleNotifyStatusToGui (false),
 	scheduleNotifyScaleMapsToGui (true),
-	defaultKey (), scale (60, defaultScale)
+	defaultKey (), scale (60, defaultScale),
+	distUni (0.0, 1.0), distBi (-1.0, 1.0)
 
 {
+	rnd.seed (time(0));
+
 	//Scan host features for URID map
 	LV2_URID_Map* m = NULL;
 	LV2_URID_Unmap* u = NULL;
@@ -103,54 +109,6 @@ void BSEQuencer::connect_port (uint32_t port, void *data)
 	}
 }
 
-/* Prepares MIDI output of the respective key for sending
- * @param frames: time.frames
- * @param status: MIDI status byte
- * @param key: key number as element of inKeys
- * @param row: pad row
- * @param chbits: BSEQencer channels as bits
- */
-bool BSEQuencer::makeMidi (const int64_t frames, const uint8_t status, const int key, const int row, uint8_t chbits)
-{
-	if ((key >= 0) && (key < int (inKeys.size)))
-	{
-		int inKeyElement = scale.getElement(inKeys[key].note);
-
-		// Allow only valid keys
-		if (inKeyElement != ENOTE)
-		{
-			Pad* pd = &inKeys[key].output[row].pad;
-			uint8_t outCh = (((uint8_t)pd->ch) & 0x0F);
-			if ((1 << (outCh - 1)) & chbits)
-			{
-				int scaleNr = controllers[SCALE];
-				int outNote;
-
-				// Drumkit: absolute MIDI notes, not input pitched
-				if (rtScaleMaps[scaleNr].elements[row] &0x100) outNote = rtScaleMaps[scaleNr].elements[row] & 0x0FF;
-
-				// Scale: relative Notes obtained from actual scale, input pitched
-				else
-				{
-					int pitch = ((controllers[CH + (outCh - 1) * CH_SIZE + PITCH]) ? inKeyElement : 0);
-					outNote = scale.getMIDInote((rtScaleMaps[scaleNr].elements[row] & 0x0FF) + pitch);
-				}
-
-				// Apply octave shift, note offset
-				outNote += pd->pitchOctave * 12 + controllers[CH + (outCh - 1) * CH_SIZE + NOTE_OFFSET];
-
-				float outVelocity = ((float)inKeys[key].velocity) * pd->velocity * controllers[CH + (outCh - 1) * CH_SIZE + VELOCITY];
-				if ((outNote >=0) && (outNote <= 127))
-				{
-					midiStack.append (frames, outCh, status, outNote, LIMIT (outVelocity, 0, 127));
-					return (status == LV2_MIDI_MSG_NOTE_ON);
-				}
-			}
-		}
-	}
-	return false;
-}
-
 /*
  * Stops MIDI output and deletes the output playing flag for the respective pads
  */
@@ -164,11 +122,11 @@ void BSEQuencer::stopMidiOut (const int64_t frames, const int key, const uint8_t
 }
 void BSEQuencer::stopMidiOut (const int64_t frames, const int key, const int row, const uint8_t chbits)
 {
-	if ((key >= 0) && (key < ((int) inKeys.size)) && inKeys[key].output[row].playing)
-	{
-		makeMidi (frames, LV2_MIDI_MSG_NOTE_OFF, key, row, chbits);
-		inKeys[key].output[row].playing = false;
-	}
+	if ((key < 0) || (key >= ((int) inKeys.size)) || (!inKeys[key].output[row].playing)) return;
+
+	Output& o = inKeys[key].output[row];
+	if (o.gate) midiStack.append (frames, o.ch, LV2_MIDI_MSG_NOTE_OFF, o.note, o.velocity);
+	o.playing = false;
 }
 
 /*
@@ -178,11 +136,64 @@ void BSEQuencer::startMidiOut (const int64_t frames, const int key, const uint8_
 {
 	for (int i = 0; i < ROWS; ++i) startMidiOut (frames, key, i, chbits);
 }
+
 void BSEQuencer::startMidiOut (const int64_t frames, const int key, const int row, const uint8_t chbits)
 {
-	if ((key >= 0) && (key < ((int) inKeys.size)) && (chbits & ((uint8_t) inKeys[key].output[row].pad.ch) & 0x0F))
+	if ((key < 0) || (key >= ((int) inKeys.size))) return;
+
+	Output& o = inKeys[key].output[row];
+	int inKeyElement = scale.getElement(inKeys[key].note);
+
+	if
+	(
+		(inKeyElement != ENOTE) &&				// Ignore invalid keys
+		((uint8_t (o.pad.ch) & 0x0F) != 0) &&			// Ignore empty pad
+		(chbits & (1 << ((uint8_t (o.pad.ch) & 0x0F) - 1))) &&	// Filter channels
+		(!o.playing)						// Ignore if note is already playing
+	)
 	{
-		inKeys[key].output[row].playing = (inKeys[key].output[row].playing || makeMidi (frames, LV2_MIDI_MSG_NOTE_ON, key, row, chbits));
+		// Set sequencer channel
+		o.ch = (uint8_t (o.pad.ch) & 0x0F) - 1;
+
+		// Set / randomize gate
+		o.gate = (distUni (rnd) < o.pad.randGate);
+
+		// Set / randomize note
+		int scaleNr = controllers[SCALE];
+		int outNote;
+
+		// Drumkit: absolute MIDI notes, not input pitched
+		if (rtScaleMaps[scaleNr].elements[row] &0x100) outNote = rtScaleMaps[scaleNr].elements[row] & 0x0FF;
+
+		// Scale: relative Notes obtained from actual scale, input pitched
+		else
+		{
+			int pitch = ((controllers[CH + o.ch * CH_SIZE + PITCH]) ? inKeyElement : 0);
+			outNote = scale.getMIDInote((rtScaleMaps[scaleNr].elements[row] & 0x0FF) + pitch);
+		}
+
+		// Apply octave shift, note offset
+		int padOctave = o.pad.pitchOctave + round (distBi (rnd) * o.pad.randOctave);
+		int padNote = o.pad.pitchNote + round (distBi (rnd) * o.pad.randNote);
+		outNote += LIMIT (padOctave, -8, 8) * 12 + LIMIT (padNote, -16, 16) + controllers[CH + o.ch * CH_SIZE + NOTE_OFFSET];
+
+		o.note = LIMIT (outNote, 0, 127);
+
+		// Set / randomize velocity
+		float padVelocity = o.pad.velocity + round (distBi (rnd) * o.pad.randVelocity);
+		float outVelocity = float (inKeys[key].velocity) * padVelocity * controllers[CH + o.ch * CH_SIZE + VELOCITY];
+
+		o.velocity = LIMIT (outVelocity, 0, 127);
+
+		// Set / randomize duration
+		float dm = fmod (o.pad.duration, 1.0);
+		if (dm == 0.0) dm = 1.0;
+		float rd = LIMIT (o.pad.randDuration, -dm, 0.0);
+		float duration = o.pad.duration * (1 + distUni (rnd) * rd / dm);
+		o.duration = LIMIT (duration, 0.0, 32.0);
+
+		if (o.gate) midiStack.append (frames, o.ch, LV2_MIDI_MSG_NOTE_ON, o.note, o.velocity);
+		o.playing = true;
 	}
 }
 
@@ -219,6 +230,44 @@ void BSEQuencer::cleanupInKeys ()
 	} while (! valid);
 }
 
+void BSEQuencer::makeAutoKey (const uint64_t last_t)
+{
+	// Exactly one inKey needed for autoplay
+	// No inKeys => create an empty preliminary key
+	if (inKeys.empty())
+	{
+		Key key = defaultKey;
+		key.note = -99;
+		inKeys.push_back (key);
+	}
+
+	// More than one inKey => shrink to one
+	while (inKeys.size > 1)
+	{
+		stopMidiOut (last_t, inKeys.size - 1, ALL_CH);
+		inKeys.pop_back ();
+	}
+
+	// Check if inKey already plays the root key
+	if (inKeys[0].note != controllers[ROOT] + controllers[SIGNATURE] + (controllers[OCTAVE] + 1) * 12)
+	{
+		stopMidiOut (last_t, 0, ALL_CH);
+		inKeys[0] = defaultKey; // stepNr = -1; direction = 1; output.pads, output.playing and jumpOff ()-initialized
+		inKeys[0].note = controllers[ROOT] + controllers[SIGNATURE] + (controllers[OCTAVE] + 1) * 12;
+		inKeys[0].velocity = 64;
+		inKeys[0].startPos = position + double (last_t) / FRAMES_PER_BEAT - (1 / STEPS_PER_BEAT);
+	}
+}
+
+void BSEQuencer::stopAutoKey (const uint64_t act_t)
+{
+	if ((inKeys.size > 0) && (inKeys[0].note == controllers[ROOT] + controllers[SIGNATURE] + (controllers[OCTAVE] + 1) * 12))
+	{
+		stopMidiOut (act_t, 0, ALL_CH);
+		inKeys.erase (&inKeys.iterator[0]);
+	}
+}
+
 bool BSEQuencer::padHasAntecessor (const int row, const int step)
 {
 	return
@@ -515,7 +564,16 @@ void BSEQuencer::runSequencer (const double startpos, const uint32_t start, cons
 										stopMidiOut (actframes, key, row, ALL_CH);
 
 										inKeys[key].output[row].pad = pads[row][newRowStepNr];
-										startMidiOut (actframes, key, row, ALL_CH);
+										if (inKeys[key].note != 0xff) startMidiOut (actframes, key, row, ALL_CH);
+									}
+
+									if
+									(
+										(pads[row][oldRowStepNr].duration > 1.0f) &&
+										(inKeys[key].output[row].playing)
+									)
+									{
+										inKeys[key].output[row].duration -= 1.0;
 									}
 
 								}
@@ -538,7 +596,7 @@ void BSEQuencer::runSequencer (const double startpos, const uint32_t start, cons
 							// Scan pads and calculate note off position
 							if (inKeys[key].output[row].playing)
 							{
-								double noteoffpos = inKeys[key].startPos + inKeys[key].output[row].pad.duration / STEPS_PER_BEAT;
+								double noteoffpos = inKeys[key].startPos + inKeys[key].output[row].duration / STEPS_PER_BEAT;
 								if ((noteoffpos >= lastpos) && (noteoffpos <= actpos))
 								{
 									int64_t noteoffframes = LIMIT (start + (noteoffpos - startpos) * FRAMES_PER_BEAT, start, end);
@@ -592,14 +650,18 @@ void BSEQuencer::run (uint32_t n_samples)
 	// Update global controllers
 	// 1. Stop MIDI out if midi_in channel, steps, play, mode or root (note/signature/octave) changed
 	bool midiStopped[NR_SEQUENCER_CHS] = {false, false, false, false};
-	if (CONTROLLER_CHANGED(MIDI_IN_CHANNEL) ||
+	if
+	(
+		CONTROLLER_CHANGED(MIDI_IN_CHANNEL) ||
 		CONTROLLER_CHANGED(NR_OF_STEPS) ||
 		CONTROLLER_CHANGED(PLAY) ||
 		CONTROLLER_CHANGED(MODE) ||
+		CONTROLLER_CHANGED(ON_KEY_PRESSED) ||
 		CONTROLLER_CHANGED(SCALE) ||
 		CONTROLLER_CHANGED(ROOT) ||
 		CONTROLLER_CHANGED(SIGNATURE) ||
-		CONTROLLER_CHANGED(OCTAVE))
+		CONTROLLER_CHANGED(OCTAVE)
+	)
 	{
 		//fprintf (stderr, "Call stopMidiOut from 'Update global controllers' at %f\n", position);
 		stopMidiOut(0, ALL_CH);
@@ -607,8 +669,14 @@ void BSEQuencer::run (uint32_t n_samples)
 	}
 
 	// 2. Stop also MIDI in if midi_in channel, steps, play or mode is changed
-	if (CONTROLLER_CHANGED(MIDI_IN_CHANNEL) || CONTROLLER_CHANGED(NR_OF_STEPS) || (CONTROLLER_CHANGED(MODE)) ||
-		(CONTROLLER_CHANGED(PLAY)))
+	if
+	(
+		CONTROLLER_CHANGED(MIDI_IN_CHANNEL) ||
+		CONTROLLER_CHANGED(NR_OF_STEPS) ||
+		CONTROLLER_CHANGED(MODE) ||
+		CONTROLLER_CHANGED(ON_KEY_PRESSED) ||
+		CONTROLLER_CHANGED(PLAY)
+	)
 	{
 		while (!inKeys.empty ()) inKeys.pop_back ();
 	}
@@ -689,7 +757,11 @@ void BSEQuencer::run (uint32_t n_samples)
 							int step = (int) pMes[i].step;
 							if ((row >= 0) && (row < ROWS) && (step >= 0) && (step < MAXSTEPS))
 							{
-								Pad pd (pMes[i].ch, pMes[i].pitchOctave, pMes[i].velocity, pMes[i].duration);
+								Pad pd
+								(
+									pMes[i].ch, pMes[i].pitchNote, pMes[i].pitchOctave, pMes[i].velocity, pMes[i].duration,
+									pMes[i].randGate, pMes[i].randNote, pMes[i].randOctave,pMes[i].randVelocity, pMes[i].randDuration
+								);
 								Pad valPad = validatePad (pd);
 								pads[row][step] = valPad;
 								if (valPad != pd)
@@ -791,15 +863,16 @@ void BSEQuencer::run (uint32_t n_samples)
 			// Host time notifications
 			else if (obj->body.otype == uris.time_Position)
 			{
-				if (controllers[MODE] == HOST_CONTROLLED)
+				if ((controllers[MODE] == HOST_CONTROLLED) || (controllers[MODE] == HOST_PLAYBACK))
 				{
 					bool scheduleStopMidi = false;
-					LV2_Atom *oBpm = NULL, *oBpb = NULL;
+					LV2_Atom *oBpm = NULL, *oBpb = NULL, *oSpeed = NULL;
 					lv2_atom_object_get
 					(
 						obj,
 						uris.time_beatsPerMinute,  &oBpm,
 						uris.time_beatsPerBar,  &oBpb,
+						uris.time_speed,  &oSpeed,
 						NULL
 					);
 
@@ -807,26 +880,31 @@ void BSEQuencer::run (uint32_t n_samples)
 					if (oBpm && (oBpm->type == uris.atom_Float) && (bpm != ((LV2_Atom_Float*)oBpm)->body))
 					{
 						bpm = ((LV2_Atom_Float*)oBpm)->body;
-						//fprintf (stderr, "BSEQuencer.lv2: bpm set to %f.\n", bpm);
-						if (controllers[MODE] == HOST_CONTROLLED) scheduleStopMidi = true;
+						scheduleStopMidi = true;
 					}
 
 					// Beats per bar changed?
 					if (oBpb && (oBpb->type == uris.atom_Float) && (beatsPerBar != ((LV2_Atom_Float*)oBpb)->body) && (((LV2_Atom_Float*)oBpb)->body > 0))
 					{
 						beatsPerBar = ((LV2_Atom_Float*)oBpb)->body;
-						//fprintf (stderr, "BSEQuencer.lv2: bpb set to %f.\n", beatsPerBar);
-						if (controllers[MODE] == HOST_CONTROLLED) scheduleStopMidi = true;
+						scheduleStopMidi = true;
+					}
+
+					// Speed changed?
+					if (oSpeed && (oSpeed->type == uris.atom_Float) && (speed != ((LV2_Atom_Float*)oSpeed)->body) && (controllers[MODE] == HOST_PLAYBACK))
+					{
+						speed = ((LV2_Atom_Float*)oSpeed)->body;
+						if (speed == 0.0f) stopAutoKey (last_t);
+						else makeAutoKey (act_t);
 					}
 
 					// Stop MIDI output for all BSEQuencer channels
-					if (scheduleStopMidi && (controllers[MODE] != AUTOPLAY))
+					if (scheduleStopMidi)
 					{
 						for (int i = 0; i < NR_SEQUENCER_CHS; ++i)
 						{
 							if (!midiStopped[i])
 							{
-								// fprintf (stderr, "Call stopMidiOut from 'Stop MIDI output for all BSEQuencer channels' at %f\n", position);
 								stopMidiOut (act_t, 1 << i);
 								midiStopped[i] = true;
 							}
@@ -846,6 +924,7 @@ void BSEQuencer::run (uint32_t n_samples)
 			if ((controllers[PLAY]) && (controllers[MODE] == HOST_CONTROLLED))
 			{
 				const uint8_t* const msg = (const uint8_t*)(ev + 1);
+				const uint8_t msize = ev->body.size;
 				uint8_t typ = lv2_midi_message_type(msg);
 				uint8_t chn = msg[0] & 0x0F;
 				uint8_t note = msg[1];
@@ -873,12 +952,40 @@ void BSEQuencer::run (uint32_t n_samples)
 							// Build new key from MIDI data
 							if (newNote)
 							{
-								Key key = defaultKey; // stepNr = -1; direction = 1; output.pads, output.playing and jumpOff ()-initialized
-								key.note = note;
-								key.velocity = msg[2];
-								key.startPos = position + double (act_t) / FRAMES_PER_BEAT - (1 / STEPS_PER_BEAT);
-								inKeys.push_back (key);
-								// fprintf (stderr, "BSEQuencer.lv2: Key on (frames: %li, note: %i, velocity: %i) at %f\n", act_t, key.note, key.velocity, key.startPos);
+								if
+								(
+									(controllers[MODE] == AUTOPLAY) ||
+									(controllers[MODE] == HOST_PLAYBACK) ||
+									(controllers[ON_KEY_PRESSED] == ON_KEY_RESTART) ||
+									(inKeys.empty())
+								)
+								{
+									Key key = defaultKey; // stepNr = -1; direction = 1; output.pads, output.playing and jumpOff ()-initialized
+									key.note = note;
+									key.velocity = msg[2];
+									key.startPos = position + double (act_t) / FRAMES_PER_BEAT - (1 / STEPS_PER_BEAT);
+									inKeys.push_back (key);
+								}
+
+								else if (controllers[ON_KEY_PRESSED] == ON_KEY_SYNC)
+								{
+									Key key = defaultKey;
+									key.note = note;
+									key.velocity = msg[2];
+									key.startPos = inKeys.back().startPos - (1 / STEPS_PER_BEAT);
+									inKeys.push_back (key);
+								}
+
+								else
+								{
+									Key key = inKeys.back();
+									key.note = note;
+									key.velocity = msg[2];
+									if (inKeys.back().note == 0xff) inKeys.back() = key;
+									else inKeys.push_back (key);
+								}
+
+								//fprintf (stderr, "BSEQuencer.lv2: Key on (frames: %li, note: %i, velocity: %i) at %f\n", act_t, key.note, key.velocity, key.startPos);
 							}
 						}
 						break;
@@ -893,7 +1000,17 @@ void BSEQuencer::run (uint32_t n_samples)
 									// fprintf (stderr, "BSEQuencer.lv2: Key off (frames: %li, note: %i, velocity: %i) at %f + frames\n", act_t, note, msg[2], position);
 
 									stopMidiOut (act_t, i, ALL_CH);
-									inKeys.erase (&inKeys.iterator[i]);
+
+									if
+									(
+										(controllers[MODE] == AUTOPLAY) ||
+										(controllers[MODE] == HOST_PLAYBACK) ||
+										(controllers[ON_KEY_PRESSED] == ON_KEY_RESTART) ||
+										(controllers[ON_KEY_PRESSED] == ON_KEY_SYNC) ||
+										(inKeys.size > 1)
+									) inKeys.erase (&inKeys.iterator[i]);
+									else inKeys[i].note = 0xff;
+
 									break;
 								}
 							}
@@ -910,7 +1027,7 @@ void BSEQuencer::run (uint32_t n_samples)
 							case LV2_MIDI_CTL_SUSTAIN:
 								for (int ch = 1; ch <= NR_SEQUENCER_CHS; ++ch)
 								{
-									midiStack.append (act_t, ch, LV2_MIDI_MSG_CONTROLLER, LV2_MIDI_CTL_SUSTAIN, msg[2]);
+									midiStack.append (act_t, ch - 1, LV2_MIDI_MSG_CONTROLLER, LV2_MIDI_CTL_SUSTAIN, msg[2]);
 								}
 								break;
 
@@ -933,7 +1050,8 @@ void BSEQuencer::run (uint32_t n_samples)
 
 							// All other MIDI signals
 							default:
-								//fprintf (stderr, "BSEQuencer.lv2: Ignored MIDI_in message in run (): #%i (%i, %i).\n", msg[0], msg[1], msg[2]);
+								//fprintf (stderr, "BSEQuencer.lv2: Ignored MIDI_in CTL message in run (): #%i (%i, %i).\n", msg[0], msg[1], msg[2]);
+								midiStack.append (act_t, 0xff, msg[0], msg[1], msg[2], msize);
 								break;
 							}
 
@@ -944,10 +1062,15 @@ void BSEQuencer::run (uint32_t n_samples)
 					// All other MIDI signals
 					default:
 						//fprintf (stderr, "BSEQuencer.lv2: Ignored MIDI_in message in run (): #%i (%i, %i).\n", msg[0], msg[1], msg[2]);
+						midiStack.append (act_t, 0xff, msg[0], msg[1], msg[2], msize);
 						break;
 					}
 				}
-				//else fprintf (stderr, "BSEQuencer.lv2: MIDI input channel filter passed MIDI_in message in run (): #%i (%i, %i).\n", msg[0], msg[1], msg[2]);
+				else
+				{
+					//fprintf (stderr, "BSEQuencer.lv2: MIDI input channel filter passed MIDI_in message in run (): #%i (%i, %i).\n", msg[0], msg[1], msg[2]);
+					midiStack.append (act_t, 0xff, msg[0], msg[1], msg[2], msize);
+				}
 			}
 		}
 
@@ -961,34 +1084,7 @@ void BSEQuencer::run (uint32_t n_samples)
 	}
 
 	// AUTOPLAY pseudo MIDI in
-	if ((controllers[PLAY]) && (controllers[MODE] == AUTOPLAY))
-	{
-		// Exactly one inKey needed for autoplay
-		// No inKeys => create an empty preliminary key
-		if (inKeys.empty())
-		{
-			Key key = defaultKey;
-			key.note = -99;
-			inKeys.push_back (key);
-		}
-
-		// More than one inKey => shrink to one
-		while (inKeys.size > 1)
-		{
-			stopMidiOut (last_t, inKeys.size - 1, ALL_CH);
-			inKeys.pop_back ();
-		}
-
-		// Check if inKey already plays the root key
-		if (inKeys[0].note != controllers[ROOT] + controllers[SIGNATURE] + (controllers[OCTAVE] + 1) * 12)
-		{
-			stopMidiOut (last_t, 0, ALL_CH);
-			inKeys[0] = defaultKey; // stepNr = -1; direction = 1; output.pads, output.playing and jumpOff ()-initialized
-			inKeys[0].note = controllers[ROOT] + controllers[SIGNATURE] + (controllers[OCTAVE] + 1) * 12;
-			inKeys[0].velocity = 64;
-			inKeys[0].startPos = position + double (last_t) / FRAMES_PER_BEAT - (1 / STEPS_PER_BEAT);
-		}
-	}
+	if ((controllers[PLAY]) && (controllers[MODE] == AUTOPLAY)) makeAutoKey (last_t);
 
 	// Update for the remainder of the cycle
 	if ((controllers[PLAY]) && (last_t < n_samples)) runSequencer (position + double (last_t) / FRAMES_PER_BEAT, last_t, n_samples);
@@ -1010,19 +1106,27 @@ LV2_State_Status BSEQuencer::state_save (LV2_State_Store_Function store, LV2_Sta
 			const LV2_Feature* const* features)
 {
 	// Store pads
-	char padDataString[0x8010] = "Matrix data:\n";
+	char padDataString[0x10010] = "Matrix data:\n";
 
 	for (int step = 0; step < MAXSTEPS; ++step)
 	{
 		for (int row = 0; row < ROWS; ++row)
 		{
-			char valueString[64];
-			int id = step * ROWS + row;
-			Pad* pd = &pads[row][step];
-			snprintf (valueString, 62, "id:%d; ch:%d; oc:%d; ve:%1.2f; du:%1.2f", id, (int) pd->ch, (int) pd->pitchOctave, pd->velocity, pd->duration);
-			if ((step < MAXSTEPS - 1) || (row < ROWS)) strcat (valueString, ";\n");
-			else strcat(valueString, "\n");
-			strcat (padDataString, valueString);
+			if (pads[row][step].ch != 0)
+			{
+				Pad* pd = &pads[row][step];
+				char valueString[128];
+				int id = step * ROWS + row;
+				snprintf
+				(
+					valueString, 126, "id:%d; ch:%d; st:%d; oc:%d; ve:%1.2f; du:%1.2f; rg:%d; rs:%d; ro:%d; rv:%1.2f; rd:%1.2f",
+					id, (int) pd->ch, (int)pd->pitchNote, (int) pd->pitchOctave, pd->velocity, pd->duration,
+					(int) pd->randGate, (int) pd->randNote, (int) pd->randOctave, pd->randVelocity, pd->randDuration
+				);
+				if ((step < MAXSTEPS - 1) || (row < ROWS)) strcat (valueString, ";\n");
+				else strcat(valueString, "\n");
+				strcat (padDataString, valueString);
+			}
 		}
 	}
 	store (handle, uris.state_pad, padDataString, strlen (padDataString) + 1, uris.atom_String, LV2_STATE_IS_POD);
@@ -1080,10 +1184,16 @@ LV2_State_Status BSEQuencer::state_restore (LV2_State_Retrieve_Function retrieve
 		// Clear all MIDI in
 		while (!inKeys.empty()) inKeys.pop_back();
 
+		// Clear all pads
+		for (int r = 0; r < ROWS; ++r)
+		{
+			for (int s = 0; s < MAXSTEPS; ++s) pads[r][s] = Pad();
+		}
+
 		// Restore pads
 		// Parse retrieved data
 		std::string padDataString = (char*) padData;
-		const std::string keywords[5] = {"id:", "ch:", "oc:", "ve:", "du:"};
+		const std::string keywords[11] = {"id:", "ch:", "st:", "oc:", "ve:", "du:", "rg:", "rs:", "ro:", "rv:", "rd:"};
 		while (!padDataString.empty())
 		{
 			// Look for next "id:"
@@ -1093,7 +1203,7 @@ LV2_State_Status BSEQuencer::state_restore (LV2_State_Retrieve_Function retrieve
 			if (strPos + 3 > padDataString.length()) break;	// Nothing more after id => end
 			padDataString.erase (0, strPos + 3);
 			int id;
-			try {id = std::stof (padDataString, &nextPos);}
+			try {id = BUtilities::stof (padDataString, &nextPos);}
 			catch  (const std::exception& e)
 			{
 				fprintf (stderr, "BSEQuencer.lv2: Restore pad state incomplete. Can't parse ID from \"%s...\"", padDataString.substr (0, 63).c_str());
@@ -1108,9 +1218,10 @@ LV2_State_Status BSEQuencer::state_restore (LV2_State_Retrieve_Function retrieve
 			}
 			int row = id % ROWS;
 			int step = id / ROWS;
+			pads[row][step] = Pad (0, 0, 0, 0, 0, 1, 0, 0, 0, 0);
 
 			// Look for pad data
-			for (int i = 1; i < 5; ++i)
+			for (int i = 1; i < 11; ++i)
 			{
 				strPos = padDataString.find (keywords[i]);
 				if (strPos == std::string::npos) continue;	// Keyword not found => next keyword
@@ -1121,7 +1232,7 @@ LV2_State_Status BSEQuencer::state_restore (LV2_State_Retrieve_Function retrieve
 				}
 				if (strPos > 0) padDataString.erase (0, strPos + 3);
 				float val;
-				try {val = std::stof (padDataString, &nextPos);}
+				try {val = BUtilities::stof (padDataString, &nextPos);}
 				catch  (const std::exception& e)
 				{
 					fprintf (stderr, "BSEQuencer.lv2: Restore padstate incomplete. Can't parse %s from \"%s...\"",
@@ -1133,11 +1244,23 @@ LV2_State_Status BSEQuencer::state_restore (LV2_State_Retrieve_Function retrieve
 				switch (i) {
 				case 1: pads[row][step].ch = val;
 						break;
-				case 2: pads[row][step].pitchOctave = val;
+				case 2: pads[row][step].pitchNote = val;
 						break;
-				case 3:	pads[row][step].velocity = val;
+				case 3: pads[row][step].pitchOctave = val;
 						break;
-				case 4:	pads[row][step].duration = val;
+				case 4:	pads[row][step].velocity = val;
+						break;
+				case 5:	pads[row][step].duration = val;
+						break;
+				case 6:	pads[row][step].randGate = val;
+						break;
+				case 7:	pads[row][step].randNote = val;
+						break;
+				case 8:	pads[row][step].randOctave = val;
+						break;
+				case 9:	pads[row][step].randVelocity = val;
+						break;
+				case 10:pads[row][step].randDuration = val;
 						break;
 				default:break;
 				}
@@ -1185,7 +1308,7 @@ LV2_State_Status BSEQuencer::state_restore (LV2_State_Retrieve_Function retrieve
 			int id = -1;
 			int scaleNr = -1;
 
-			try {id = std::stof (scaleDataString, &nextPos);}
+			try {id = BUtilities::stof (scaleDataString, &nextPos);}
 			catch  (const std::exception& e)
 			{
 				fprintf (stderr, "BSEQuencer.lv2: Restore scale map state incomplete. Can't parse ID from \"%s...\"", scaleDataString.substr (0, 63).c_str());
@@ -1349,10 +1472,15 @@ Pad BSEQuencer::validatePad (Pad pad)
 	return Pad
 	(
 		validateValue (pad.ch, {0, 255, 1}),
+		validateValue (pad.pitchNote, controllerLimits[SELECTION_NOTE]),
 		validateValue (pad.pitchOctave, controllerLimits[SELECTION_OCTAVE]),
 		validateValue (pad.velocity, controllerLimits[SELECTION_VELOCITY]),
-		LIMIT (pad.duration, 0.0, 32.0)
-		//validateValue (pad.duration, controllerLimits[SELECTION_DURATION])
+		LIMIT (pad.duration, 0.0, 32.0),
+		validateValue (pad.randGate, controllerLimits[SELECTION_GATE_RAND]),
+		validateValue (pad.randNote, controllerLimits[SELECTION_NOTE_RAND]),
+		validateValue (pad.randOctave, controllerLimits[SELECTION_OCTAVE_RAND]),
+		validateValue (pad.randVelocity, controllerLimits[SELECTION_VELOCITY_RAND]),
+		validateValue (pad.randDuration, controllerLimits[SELECTION_DURATION_RAND])
 	);
 }
 
@@ -1362,7 +1490,11 @@ Pad BSEQuencer::validatePad (Pad pad)
 bool BSEQuencer::padMessageBufferAppendPad (int row, int step, Pad pad)
 {
 	PadMessage end = PadMessage (ENDPADMESSAGE);
-	PadMessage msg = PadMessage (step, row, pad.ch, pad.pitchOctave, pad.velocity, pad.duration);
+	PadMessage msg = PadMessage
+	(
+		step, row, pad.ch, pad.pitchNote, pad.pitchOctave, pad.velocity, pad.duration,
+		pad.randGate, pad.randNote, pad.randOctave, pad.randVelocity, pad.randDuration
+	);
 
 	for (int i = 0; i < MAXSTEPS * ROWS; ++i)
 	{
@@ -1387,7 +1519,11 @@ void BSEQuencer::padMessageBufferAllPads ()
 		for (int j = 0; j < ROWS; ++j)
 		{
 			Pad* pd = &(pads[j][i]);
-			padMessageBuffer[i * ROWS + j] = PadMessage (i, j, pd->ch, pd->pitchOctave, pd->velocity, pd->duration);
+			padMessageBuffer[i * ROWS + j] = PadMessage
+			(
+				i, j, pd->ch, pd->pitchNote, pd->pitchOctave, pd->velocity, pd->duration,
+				pd->randGate, pd->randNote, pd->randOctave, pd->randVelocity, pd->randDuration
+			);
 		}
 	}
 }
@@ -1428,25 +1564,28 @@ void BSEQuencer::notifyStatusToGui ()
 	int8_t size = scale.getSize ();
 	for (size_t i = 0; i < inKeys.size; ++i)
 	{
-		int8_t element = scale.getElement(inKeys[i].note);
-
-		// Only valid keys
-		if ((element != ENOTE) && (size != ENOTE))
+		if (inKeys[i].note != 0xff)
 		{
-			// Set notebits
-			notebits = notebits | (1 << (element - int (floor (double (element) / double (size)) * double (size))));
+			int8_t element = scale.getElement(inKeys[i].note);
 
-			for (int row = 0; row < ROWS; ++row)
+			// Only valid keys
+			if ((element != ENOTE) && (size != ENOTE))
 			{
-				if ((inKeys[i].stepNr >= 0) && (inKeys[i].output[row].stepOffset != HALT_STEP))
+				// Set notebits
+				notebits = notebits | (1 << (element - int (floor (double (element) / double (size)) * double (size))));
+
+				for (int row = 0; row < ROWS; ++row)
 				{
-					int stepNr = (inKeys[i].stepNr + inKeys[i].output[row].stepOffset) % ((int)controllers[NR_OF_STEPS]);
+					if ((inKeys[i].stepNr >= 0) && (inKeys[i].output[row].stepOffset != HALT_STEP))
+					{
+						int stepNr = (inKeys[i].stepNr + inKeys[i].output[row].stepOffset) % ((int)controllers[NR_OF_STEPS]);
 
-					// Set cursorbits
-					cursorbits[stepNr] = (cursorbits[stepNr] | (1 << row));
+						// Set cursorbits
+						cursorbits[stepNr] = (cursorbits[stepNr] | (1 << row));
 
-					// Set chbits
-					if (((int)inKeys[i].output[row].pad.ch) & 0x0F) chbits = (chbits | (1 << (((int)inKeys[i].output[row].pad.ch - 1) & 0x0F)));
+						// Set chbits
+						if (((int)inKeys[i].output[row].pad.ch) & 0x0F) chbits = (chbits | (1 << (((int)inKeys[i].output[row].pad.ch - 1) & 0x0F)));
+					}
 				}
 			}
 		}
@@ -1498,12 +1637,12 @@ void BSEQuencer::notifyMidi ()
 	{
 		MidiData& midiData = midiStack[i];
 		// ch -> MIDI channel
-		int channel = controllers[CH + (midiData.ch - 1) * CH_SIZE + MIDI_CHANNEL] - 1;
+		int channel = (midiData.ch < NR_SEQUENCER_CHS ? controllers[CH + midiData.ch * CH_SIZE + MIDI_CHANNEL] - 1 : 0);
 
 		// compose MIDI message block
 		LV2_Atom midiatom;
 		midiatom.type = uris.midi_Event;
-		midiatom.size = 3;
+		midiatom.size = midiData.size;
 
 		uint8_t msg[3];
 		msg[0] = midiData.status + channel;
@@ -1513,8 +1652,8 @@ void BSEQuencer::notifyMidi ()
 		// send MIDI message
 		if (!lv2_atom_forge_frame_time (&output_forge, midiData.frames)) return;
 		if (!lv2_atom_forge_raw (&output_forge, &midiatom, sizeof (LV2_Atom))) return;
-		if (!lv2_atom_forge_raw (&output_forge, &msg, 3)) return;
-		lv2_atom_forge_pad (&output_forge, sizeof (LV2_Atom) + 3);
+		if (!lv2_atom_forge_raw (&output_forge, &msg, midiatom.size)) return;
+		lv2_atom_forge_pad (&output_forge, sizeof (LV2_Atom) + midiatom.size);
 	}
 }
 
@@ -1542,13 +1681,13 @@ static LV2_Handle instantiate (const LV2_Descriptor* descriptor, double samplera
 static void connect_port (LV2_Handle instance, uint32_t port, void *data)
 {
 	BSEQuencer* inst = (BSEQuencer*) instance;
-	inst->connect_port (port, data);
+	if (inst) inst->connect_port (port, data);
 }
 
 static void run (LV2_Handle instance, uint32_t n_samples)
 {
 	BSEQuencer* inst = (BSEQuencer*) instance;
-	inst->run (n_samples);
+	if (inst) inst->run (n_samples);
 }
 
 static LV2_State_Status state_save(LV2_Handle instance, LV2_State_Store_Function store, LV2_State_Handle handle, uint32_t flags,
@@ -1565,20 +1704,20 @@ static LV2_State_Status state_restore(LV2_Handle instance, LV2_State_Retrieve_Fu
            const LV2_Feature* const* features)
 {
 	BSEQuencer* inst = (BSEQuencer*)instance;
-	inst->state_restore (retrieve, handle, flags, features);
+	if (inst) inst->state_restore (retrieve, handle, flags, features);
 	return LV2_STATE_SUCCESS;
 }
 
 static void activate (LV2_Handle instance)
 {
 	BSEQuencer* inst = (BSEQuencer*)instance;
-	inst->activate ();
+	if (inst) inst->activate ();
 }
 
 static void cleanup (LV2_Handle instance)
 {
 	BSEQuencer* inst = (BSEQuencer*) instance;
-	delete inst;
+	if (inst) delete inst;
 }
 
 
